#!/usr/bin/env python
"""
_FindDASToUpload_

Oracle version of DBS3Buffer.FindDASToUpload
"""

from WMComponent.DBS3Buffer.MySQL.FindDASToUpload import FindDASToUpload as MySQLFindDASToUpload

class FindDASToUpload(MySQLFindDASToUpload):
    pass



#!/usr/bin/env python
"""
_UpdateDataset_

Oracle implementation of DBS3Buffer.UpdateDataset

Created on May 15, 2013

@author: dballest
"""

from WMComponent.DBS3Buffer.MySQL.UpdateDataset import UpdateDataset as MySQLUpdateDataset

class UpdateDataset(MySQLUpdateDataset):
    pass



"""
File       : MongoDB.py
Description: Provides a wrapper class for MongoDB
"""

# futures
from __future__ import division, print_function
from builtins import str, object

try:
    import mongomock
except ImportError:
    # this library should only be required by unit tests
    mongomock = None

from pymongo import MongoClient, errors, IndexModel


class MongoDB(object):
    """
    A simple wrapper class for creating a connection to a MongoDB instance
    """
    def __init__(self, database=None, server=None, port=None, replicaset=None,
                 create=False, collections=None, testIndexes=False,
                 logger=None, mockMongoDB=False, **kwargs):
        """
        :databases:   A database Name to connect to
        :server:      The server url (see https://docs.mongodb.com/manual/reference/connection-string/)
        :port:        Server port
        :create:      A flag to trigger a database creation (if missing) during
                      object construction, together with collections if present.
        :collections: A list of tuples describing collections with indexes -
                      the first element is considered the collection name, all
                      the rest elements are considered as indexes
        :testIndexes: A flag to trigger index test and eventually to create them
                      if missing (TODO)
        :logger:      Logger
        """
        self.server = server # '127.0.0.1'
        self.port = port # 8230
        self.logger = logger
        self.mockMongoDB = mockMongoDB
        if mockMongoDB and mongomock is None:
            msg = "You are trying to mock MongoDB, but you do not have mongomock in the python path."
            self.logger.critical(msg)
            raise ImportError(msg)
        try:
            if mockMongoDB:
                self.client = mongomock.MongoClient()
                self.logger.info("NOTICE: MongoDB is set to use mongomock, instead of real database.")
            elif replicaset:
                self.client = MongoClient(self.server, self.port, replicaset=replicaset, **kwargs )
            else:
                self.client = MongoClient(self.server, self.port, **kwargs)
            self.client.server_info()
        except Exception as ex:
            msg = "Could not connect to MongoDB server: %s\n%s" % (self.server, str(ex))
            msg += "Giving up Now."
            self.logger.error(msg)
            raise ex
        self.create = create
        self.testIndexes = testIndexes
        self.dbName = database
        self.collections = collections or []

        self._dbConnect(database)

        if self.create and self.collections:
            for collection in self.collections:
                self._collCreate(collection, database)

        if self.testIndexes and self.collections:
            for collection in self.collections:
                self._indexTest(collection[0], collection[1])

    def _indexTest(self, collection, index):
        pass

    def _collTest(self, coll, db):
        # self[db].list_collection_names()
        pass

    def collCreate(self, coll):
        """
        A public method for _collCreate
        """
        self._collCreate(coll, self.database)

    def _collCreate(self, coll, db):
        """
        A function used to explicitly create a collection with the relevant
        indexes - used to avoid the Lazy Creating from MongoDB and eventual issues
        in case we end up with no indexed collection, especially ones missing
        the (`unique` index parameter)
        :coll: A tuple describing one collection with indexes -
               The first element is considered to be the collection name, and all
               the rest of the elements are considered to be indexes.
               The indexes must be of type IndexModel. See pymongo documentation:

               https://api.mongodb.com/python/current/api/pymongo/collection.html#pymongo.collection.Collection.create_index

        :db:   The database name for the collection
        """

        collName = coll[0]
        collIndexes = list(coll[1:])
        try:
            self.client[db].create_collection(collName)
        except errors.CollectionInvalid:
            # this error is thrown in case of an already existing collection
            msg = "Collection '{}' Already exists in database '{}'".format(coll, db)
            self.logger.warning(msg)

        if collIndexes:
            for index in collIndexes:
                if not isinstance(index, IndexModel):
                    msg = "ERR: Bad Index type for collection %s" % collName
                    raise errors.InvalidName
            try:
                self.client[db][collName].create_indexes(collIndexes)
            except Exception as ex:
                msg = "Failed to create indexes on collection: %s\n%s" % (collName, str(ex))
                self.logger.error(msg)
                raise ex

    def _dbTest(self, db):
        """
        Tests database connection.
        """
        # Test connection (from mongoDB documentation):
        # https://api.mongodb.com/python/3.4.0/api/pymongo/mongo_client.html
        try:
            # The 'ismaster' command is cheap and does not require auth.
            self.client.admin.command('ismaster')
        except errors.ConnectionFailure as ex:
            msg = "Server not available: %s" % str(ex)
            self.logger.error(msg)
            raise ex

        # Test for database existence
        if db not in self.client.list_database_names():
            msg = "Missing MongoDB databases: %s" % db
            self.logger.error(msg)
            raise errors.InvalidName

    def _dbCreate(self, db):
        # creating an empty collection in order to create the database
        _initColl = self.client[db].create_collection('_initCollection')
        _initColl.insert_one({})
        # NOTE: never delete the _initCollection if you want the database to persist
        # self.client[db].drop_collection('_initCollection')

    def dbConnect(self):
        """
        A public method for _dbConnect
        """
        self._dbConnect(self.database)

    def _dbConnect(self, db):
        """
        The function to be used for the initial database connection creation and testing
        """
        try:
            setattr(self, db, self.client[db])
            if not self.mockMongoDB:
                self._dbTest(db)
        except errors.ConnectionFailure as ex:
            msg = "Could not connect to MongoDB server for database: %s\n%s\n" % (db, str(ex))
            msg += "Giving up Now."
            self.logger.error(msg)
            raise ex
        except errors.InvalidName as ex:
            msg = "Could not connect to a missing MongoDB databases: %s\n%s" % (db, str(ex))
            self.logger.error(msg)
            if self.create:
                msg = "Trying to create: %s" % db
                self.logger.error(msg)
                try:
                    # self._dbCreate(getattr(self, db))
                    self._dbCreate(db)
                except Exception as ex:
                    msg = "Could not create MongoDB databases: %s\n%s\n" % (db, str(ex))
                    msg += "Giving up Now."
                    self.logger.error(msg)
                    raise ex
                try:
                    self._dbTest(db)
                except Exception as ex:
                    msg = "Second failure while testing %s\n%s\n" % (db, str(ex))
                    msg += "Giving up Now."
                    self.logger.error(msg)
                    raise ex
                msg = "Database %s successfully created" % db
                self.logger.error(msg)
        except Exception as ex:
            msg = "General Exception while trying to connect to : %s\n%s" % (db, str(ex))
            self.logger.error(msg)
            raise ex



#!/bin/env python

"""
_ParseXMLFile_

This holds the methods used to take an xmlFilename and return a tree structure.
Used for the expat xml parsers

"""

from builtins import next, str, object
from future.utils import viewitems

import xml.parsers.expat

class Node(object):
    """
    _Node_

    Really simple DOM like container to simplify parsing the XML file
    and formatting the character data without all the whitespace guff

    """
    def __init__(self, name, attrs):
        self.name = str(name)
        self.attrs = {}
        self.text = None
        for k, v in viewitems(attrs):
            self.attrs.__setitem__(str(k), str(v))
        self.children = []

    def __str__(self):

        result = " %s %s \"%s\"\n" % (self.name, self.attrs, self.text)
        for child in self.children:
            result += str(child)
        return result



def coroutine(func):
    """
    _coroutine_

    Decorator method used to prime coroutines

    """
    def start(*args,**kwargs):
        cr = func(*args,**kwargs)
        next(cr)
        return cr
    return start


def xmlFileToNode(reportFile):
    """
    _xmlFileToNode_

    Use expat and the build coroutine to parse the XML file and build
    a node structure

    """
    node = Node("JobReports", {})
    expat_parse(open(reportFile, 'rb'),
                build(node))
    return node


def expat_parse(f, target):
    """
    _expat_parse_

    Expat based XML parsing that feeds a node building coroutine

    """
    parser = xml.parsers.expat.ParserCreate()
    #parser.buffer_size = 65536
    parser.buffer_text = True

    # a leftover from the py2py3 migration - TO BE REMOVED
    # parser.returns_unicode = False
    parser.StartElementHandler = \
       lambda name,attrs: target.send(('start',(name,attrs)))
    parser.EndElementHandler = \
       lambda name: target.send(('end',name))
    parser.CharacterDataHandler = \
       lambda data: target.send(('text',data))
    parser.ParseFile(f)


@coroutine
def build(topNode):
    """
    _build_

    Node structure builder that is fed from the expat_parse method

    """
    nodeStack = [topNode]
    charCache = []
    while True:
        event, value = (yield)
        if event == "start":
            charCache = []
            newnode = Node(value[0], value[1])
            nodeStack[-1].children.append(newnode)
            nodeStack.append(newnode)

        elif event == "text":
            charCache.append(value)

        else: # end
            nodeStack[-1].text = str(''.join(charCache)).strip()
            nodeStack.pop()
            charCache = []



#!/usr/bin/env python
"""
Retrieve basic information about the worker threads such
that it can be pushed to the monitoring tool
"""
from __future__ import division

from WMCore.Database.DBFormatter import DBFormatter


class MonitorWorkers(DBFormatter):
    sql = """SELECT name, last_updated, state, poll_interval, cycle_time
               FROM wm_workers ORDER BY name"""

    def execute(self, conn=None, transaction=False):
        result = self.dbi.processData(self.sql, conn=conn, transaction=transaction)
        return self.formatDict(result)




from builtins import object
import threading

from sqlalchemy import create_engine
from sqlalchemy import __version__ as sqlalchemy_version
from WMCore.Database.Dialects import MySQLDialect
from WMCore.Database.Dialects import OracleDialect

class DBFactory(object):

    # class variable
    _engineMap = {}
    _defaultEngineParams = {"convert_unicode" : True,
                            "pool_recycle": 7200}

    def __init__(self, logger, dburl=None, options={}):
        self.logger = logger
        # get the engine parameter from option
        if 'engine_parameters' in options:
            self._defaultEngineParams.update(options['engine_parameters'])
            del options['engine_parameters']

        if dburl:
            self.dburl = dburl
        else:
            #This will be deprecated.
            """
            Need to make the dburl here. Possible formats are:

            mysql://host/database
            mysql://username@host/database
            mysql://username:password@host:port/database

            oracle://username:password@tnsName
            oracle://username:password@host:port/sidname

            """
            hostjoin = ''
            if 'dialect' in options:
                self.dburl = '%s://' % options['dialect']
                del options['dialect']
            if 'user' in options:
                self.dburl = '%s%s' % (self.dburl, options['user'])
                hostjoin='@'
                del options['user']
            if 'username' in options:
                self.dburl = '%s%s' % (self.dburl, options['username'])
                hostjoin='@'
                del options['username']
            if 'passwd' in options:
                self.dburl = '%s:%s' % (self.dburl, options['passwd'])
                del options['passwd']
            if 'password' in options:
                self.dburl = '%s:%s' % (self.dburl, options['password'])
                del options['password']
            if 'tnsName' in options:
                self.dburl = '%s%s%s' % (self.dburl, hostjoin, options['tnsName'])
                del options['tnsName']
            elif 'host' in options:
                self.dburl = '%s%s%s' % (self.dburl, hostjoin, options['host'])
                del options['host']
                if 'port' in options:
                    self.dburl = '%s:%s' % (self.dburl, options['port'])
                    del options['port']
            if 'host' in options:
                self.dburl = '%s/%s' % (self.dburl, options['host'])
                del options['host']
            if 'database' in options:
                self.dburl = '%s/%s' % (self.dburl, options['database'])
                del options['database']
            elif 'sid' in options:
                self.dburl = '%s/%s' % (self.dburl, options['sid'])
                del options['sid']

        if self.dburl.split(':')[0].lower() == "http":

            self.engine = None
            self.dia = None

        else:
            self.engine = self._engineMap.setdefault(self.dburl,
                                                     create_engine(self.dburl,
                                                                   connect_args=options,
                                                                   **self._defaultEngineParams)
                                                     )
            self.dia = self.engine.dialect

        self.lock = threading.Condition()


    def connect(self):
        self.lock.acquire()

        if self.engine:

            self.logger.debug("Using SQLAlchemy v.%s" % sqlalchemy_version)

            if isinstance(self.dia, MySQLDialect):
                from WMCore.Database.MySQLCore import MySQLInterface as DBInterface
            else:
                from WMCore.Database.DBCore import DBInterface
            # we instantiate within the lock so we can safely return the local instance.
            dbInterface =  DBInterface(self.logger, self.engine)

        else:
            dbInterface =  None

        self.lock.release()
        return dbInterface



#!/usr/bin/env python
"""
Oracle implementation of File.SetParentageByJob

Make the parentage link between a file and all the inputs of a given job
"""




from WMCore.WMBS.MySQL.Files.SetParentageByJob import SetParentageByJob as MySQLSetParentageByJob

class SetParentageByJob(MySQLSetParentageByJob):
    pass



#!/usr/bin/env python
"""
_KillWorkflow_

MySQL implementation of Jobs.KillWorkflow
"""

from WMCore.Database.DBFormatter import DBFormatter

class KillWorkflow(DBFormatter):
    """
    _KillWorkflow_

    Find all jobs that belong don't belong to Cleanup and LogCollect
    subscriptions and return their state and id.
    """
    sql = """SELECT wmbs_job.id, wmbs_job_state.name AS state,
               wmbs_job.retry_count AS retry_count, wmbs_job.name AS name
                    FROM wmbs_workflow
               INNER JOIN wmbs_subscription ON
                 wmbs_workflow.id = wmbs_subscription.workflow AND
                 wmbs_subscription.subtype IN
                  (SELECT id FROM wmbs_sub_types
                   WHERE name != 'Cleanup' AND name != 'LogCollect')
               INNER JOIN wmbs_jobgroup ON
                 wmbs_subscription.id = wmbs_jobgroup.subscription
               INNER JOIN wmbs_job ON
                 wmbs_jobgroup.id = wmbs_job.jobgroup AND
                 wmbs_job.state IN
                   (SELECT id FROM wmbs_job_state
                    WHERE name != 'complete' AND name != 'success' AND
                          name != 'cleanout' AND name != 'exhausted')
               INNER JOIN wmbs_job_state ON
                 wmbs_job.state = wmbs_job_state.id
             WHERE wmbs_workflow.name = :workflowname"""

    def execute(self, workflowName, conn = None, transaction = False):
        results = self.dbi.processData(self.sql, {"workflowname": workflowName},
                                       conn = conn, transaction = transaction)
        return self.formatDict(results)



#!/usr/bin/env python
"""
_MarkDatasetSubscribed_

Oracle implementation of MarkDatasetSubscribed.
"""

from __future__ import division

from WMComponent.RucioInjector.Database.MySQL.MarkDatasetSubscribed import MarkDatasetSubscribed as MySQLBase


class MarkDatasetSubscribed(MySQLBase):
    pass



#
# This is an example configuration which loads the documentation classes for
# the webtools package and shows you how to configure the various different
# classes available. Your application should have it's own configuration in it's
# CVS area and not use this, other than as a guideline. Applications deployed at
# CERN will need to meet the CERN web service SLA:
#     https://twiki.cern.ch/twiki/bin/view/CMS/DMWTServiceLevelAgreement
# This includes committing configuration files to appropriate locations in CVS.
#
from WMCore.Configuration import Configuration
from WMCore.WMBase import getWMBASE
import os.path
import logging
config = Configuration()

# This component has all the configuration of CherryPy
config.component_('Webtools')
# We could change port, set logging etc here like so:
#config.Webtools.port = 8011
#config.Webtools.show_tracebacks = True
config.Webtools.autoreload = True
config.Webtools.environment = 'development'
config.Webtools.log_screen = True
config.Webtools.error_log_level = logging.INFO
#config.Webtools.thread_pool = 10
# etc. Check Root.py for all configurables
# The above short-hand can be replaced with explicit namespaced configuration
# variables as described in http://www.cherrypy.org/wiki/ConfigAPI
# for example
config.Webtools.section_('server')
config.Webtools.server.socket_timeout = 30
# Shorthand configurations take precedence over explicit ones, e.g. if you have
#config.Webtools.server.socket_port = 8010
#config.Webtools.port = 8011
# your server will start on 8011
# This is the application
config.Webtools.application = 'WebtoolsDocs'

# This is the config for the application
config.component_('WebtoolsDocs')
# Define the default location for templates for the app
config.WebtoolsDocs.templates = os.path.join(getWMBASE(), 'src/templates/WMCore/WebTools')
config.WebtoolsDocs.admin = 'your@email.com'
config.WebtoolsDocs.title = 'CMS WMCore/WebTools Documentation'
config.WebtoolsDocs.description = 'Documentation on the WMCORE/WebTools'

# If we want an application to run on multiple for multiple instances confiure
# it as follows
config.WebtoolsDocs.instances = ['foo', 'bar']
# We could define the class that is the applications index
#config.WebtoolsDocs.index = 'welcome'
# but instead we'll leave it blank and use the default (Welcome.py) which
# inspects the pages that are loaded and auto-generates a page based on the
# classes doc strings. You can hide a view from the welcome page by setting
# hidden=True in its configuration - useful for "admin" pages.

# Views are all pages
config.WebtoolsDocs.section_('views')
# These are all the active pages that Root.py should instantiate
active = config.WebtoolsDocs.views.section_('active')
# This is the Security config the application will use
config.component_('SecurityModule')
# You can turn off security by setting
config.SecurityModule.dangerously_insecure = True
# There should be a proper HMAC key file here, use this file as an
# example
#config.SecurityModule.key_file = os.path.join(getWMBASE(), 'src/python/WMCore/WebTools/DefaultConfig.py')
#
# I could secure all the pages in the web app to these settings using a default
# configuration. This can still be over-ridden on a per page basis.
#default = config.SecurityModule.section_('default')
#default.role = ''
#default.group = ''
#default.site = ''

# The section name is also the location the class will be located
# e.g. http://localhost:8080/documentation
active.section_('documentation')
# The class to load for this view/page
active.documentation.object = 'WMCore.WebTools.Documentation'
# I could add a variable to the documenation object if I wanted to as follows:
# active.documentation.foo = 'bar'

# I can reuse a class
active.section_('secretdocumentation')
active.secretdocumentation.object = 'WMCore.WebTools.Documentation'
# I don't want the world to see the secret documents on the welcome page.
active.secretdocumentation.hidden = True

# I can use an openID secured class
active.section_('securedocumentation')
active.securedocumentation.object = 'WMCore.WebTools.SecureDocumentation'

#active.section_('welcome')
#active.welcome.object = 'WMCore.WebTools.Welcome'

# These are pages in "maintenance mode" - to be completed
maint = config.WebtoolsDocs.views.section_('maintenance')

# This is how you would configure a RESTful service
# You need to install py2-sqlalchemy to be able to use it. Put it on the
# spec file of your webtools package
#active.section_('rest')
#active.rest.object = 'WMCore.WebTools.RESTApi'
#active.rest.templates =os.path.join(WMCore.WMInit.getWMBASE(), '/src/templates/WMCore/WebTools/' )
# for more option check
# http://www.sqlalchemy.org/docs/reference/sqlalchemy/connections.html
#active.rest.database.engineParameters = {'pool_size': 10, 'max_overflow': 10, 'pool_timeout': 30}
#active.rest.section_('model')
#active.rest.model.object = 'RESTModel'
#active.rest.section_('formatter')
#active.rest.formatter.object = 'RESTFormatter'
# You could override the templates/database here, for instance:
#active.rest.formatter.templates = os.path.join(WMCore.WMInit.getWMBASE(), '/src/templates/WMCore/WebTools/')



#!/usr/bin/env python
"""
MySQL implementation of File.SetParentageByMergeJob

Make the parentage link between a file and all the inputs of a given job
"""




import logging

from WMCore.Database.DBFormatter import DBFormatter

class SetParentageByMergeJob(DBFormatter):
    """
    Make the parentage link between a file and all the inputs of a given job
    """


    sql = """INSERT INTO wmbs_file_parent (child, parent)
             SELECT DISTINCT wmbs_file_details.id, wmbs_job_assoc.fileid
             FROM wmbs_file_details, wmbs_job_assoc
             INNER JOIN wmbs_job ON
                 wmbs_job_assoc.job = wmbs_job.id
             INNER JOIN wmbs_jobgroup ON
                 wmbs_jobgroup.id = wmbs_job.jobgroup
             LEFT OUTER JOIN wmbs_sub_files_failed ON
                 wmbs_jobgroup.subscription = wmbs_sub_files_failed.subscription
                 AND wmbs_sub_files_failed.fileid = wmbs_job_assoc.fileid
             WHERE wmbs_job_assoc.job  = :jobid
             AND wmbs_file_details.lfn = :child
             AND wmbs_sub_files_failed.fileid IS NULL
    """


    def execute(self, binds, conn = None, transaction = False):
        """
        Expect binds of form {'jobid', 'child'}

        """

        result = self.dbi.processData(self.sql, binds,
                         conn = conn, transaction = transaction)

        return



#!/usr/bin/env python
# pylint: disable=E1101
"""
_WMStep_

Basic unit of executable work within a task.
Equivalent of a PayloadNode in the old production system WorkflowSpec

"""



from WMCore.WMSpec.ConfigSectionTree import ConfigSectionTree, TreeHelper
from WMCore.WMSpec.Steps.StepFactory import getStepTypeHelper


class WMStepHelper(TreeHelper):
    """
    _WMStepHelper_

    Utils, methods and functions for manipulating the data in a WMStep

    """
    def __init__(self, stepInstance):
        TreeHelper.__init__(self, stepInstance)

    def name(self):
        return self.data._internal_name

    def setStepType(self, stepType):
        """
        _setStepType_

        Set the type of the step.
        """
        self.data.stepType = stepType

    def stepType(self):
        """
        _stepType_

        Retrieve the step type.
        """
        return self.data.stepType

    def getNumberOfCores(self):
        """
        _getNumberOfCores_

        Return the number of cores for the step in question
        """
        try:
            return int(self.data.application.multicore.numberOfCores)
        except Exception:
            return 1

    def getNumberOfStreams(self):
        """
        _getNumberOfStreams_

        Return the number of event streams for the step in question
        """
        try:
            return int(self.data.application.multicore.eventStreams)
        except Exception:
            return 0

    def getGPURequired(self):
        """
        Return whether GPU is required or not for this step object
        """
        if hasattr(self.data.application, "gpu"):
            if hasattr(self.data.application.gpu, "gpuRequired"):
                return self.data.application.gpu.gpuRequired
        return None

    def getGPURequirements(self):
        """
        Return the GPU requirements for this step object
        """
        if hasattr(self.data.application, "gpu"):
            if hasattr(self.data.application.gpu, "gpuRequirements"):
                return self.data.application.gpu.gpuRequirements
        return None

    def addStep(self, stepName):
        """
        _addStep_

        Add a new step with the name provided to this step as a child
        """
        node = WMStepHelper(WMStep(stepName))
        self.addNode(node)
        return node

    def addTopStep(self, stepName):
        """
        _addTopStep_

        Add a new step with the name provided to this step as a child.  This
        will be the first top step of all the children.
        """
        node = WMStepHelper(WMStep(stepName))
        self.addTopNode(node)
        return node

    def getStep(self, stepName):
        """
        _getStep_

        Retrieve the named step and wrap it in a helper

        """
        node = self.getNode(stepName)
        if node == None:
            return None
        return WMStepHelper(node)

    def setUserDN(self, userDN):
        """
        _setUserDN_

        Set the user DN

        """
        self.data.userDN = userDN

    def setAsyncDest(self, asyncDest):
        """
        _setAsyncDest_

        Set the async. destination

        """
        self.data.asyncDest = asyncDest

    def setUserRoleAndGroup(self, owner_vogroup, owner_vorole):
        """
        _setUserRoleAndGroup_

        Set user group and role.

        """
        self.data.owner_vogroup = owner_vogroup
        self.data.owner_vorole = owner_vorole

    def getTypeHelper(self):
        """
        _getTypeHelper_

        Get a step type specific helper for this object using the StepFactory

        """
        return getStepTypeHelper(self.data)


    def addOverride(self, override, overrideValue):
        """
        _addOverride_

        Add overrides for use in step executors
        """

        setattr(self.data.override, override, overrideValue)

        return



    def getOverrides(self):
        """
        _getOverrides_

        Get overrides for use in executors
        """

        return self.data.override.dictionary_()

    def getOutputModule(self, moduleName):
        """
        _getOutputModule_

        Get an output module from the step
        Return None if non-existant
        """

        if hasattr(self.data.output, 'modules'):
            if hasattr(self.data.output.modules, moduleName):
                return getattr(self.data.output.modules, moduleName)

        return None

    def setIgnoredOutputModules(self, moduleList):
        """
        _setIgnoredOutputModules_

        Set a list of output modules to be ignored,
        only CMSSW steps will use this
        """

        self.data.output.ignoredModules = moduleList
        return

    def setNewStageoutOverride(self, newValue):
        """
        A toggle for steps to use old or new stageout code
        """
        self.data.newStageout = newValue

    def getNewStageoutOverride(self):
        """
        A toggle for steps to use old or new stageout code
        """
        if hasattr(self.data, 'newStageout'):
            return self.data.newStageout
        else:
            return False

    def getIgnoredOutputModules(self):
        """
        _ignoreOutputModules_

        Get a list of output modules to be ignored,
        if the attribute is not set then return an empty list
        """

        if hasattr(self.data.output, 'ignoredModules'):
            return self.data.output.ignoredModules
        return []

    def getUserSandboxes(self):
        if hasattr(self.data, 'user'):
            if hasattr(self.data.user, 'inputSandboxes'):
                return self.data.user.inputSandboxes
        return []

    def getUserFiles(self):
        if hasattr(self.data, 'user'):
            if hasattr(self.data.user, 'userFiles'):
                return self.data.user.userFiles
        return []

    def setErrorDestinationStep(self, stepName):
        """
        _setErrorDestinationStep_

        In case of error, give the name of the step that
        the execute process should go to.
        """

        self.data.errorDestinationStep = stepName
        return

    def getErrorDestinationStep(self):
        """
        _getErrorDestinationStep_

        In case of error, get the step that should be
        next in the process
        """

        return getattr(self.data, 'errorDestinationStep', None)


    def getConfigInfo(self):
        """
        _getConfigInfo_

        Get information about the config cache location
        """

        cacheUrl = getattr(self.data.application.configuration, 'configCacheUrl', None)
        cacheDb  = getattr(self.data.application.configuration, 'cacheName', None)
        configId = getattr(self.data.application.configuration, 'configId', None)

        return cacheUrl, cacheDb, configId


    def listAnalysisFiles(self):
        """
        _listAnalysisFiles_

        retrieve list of output module names

        """
        if hasattr(self.data.output, "analysisFiles"):
            return list(self.data.output.analysisFiles.dictionary_())

        return []


    def getAnalysisFile(self, name):
        """
        _getAnalysisFile_

        retrieve the data structure for an analysis file by name
        None if not found
        """
        return getattr(self.data.output.analysisFiles, name, None)

    def getConfigCacheID(self):
        """
        _getConfigCacheID_

        If we have a configCacheID return it, otherwise return None
        """
        return getattr(self.data.application.configuration, 'retrieveConfigUrl', None)



class WMStep(ConfigSectionTree):
    """
    _WMStep_

    Container for an executable unit within a Task

    """
    def __init__(self, name):
        ConfigSectionTree.__init__(self, name)
        self.objectType = self.__class__.__name__
        self.stepType = None

        self.section_("application")
        self.application.section_("controls")
        self.application.section_("configuration")

        self.section_("input")
        self.input.section_("links")
        self.section_("output")
        self.section_("sandbox")

        self.section_("emulator")
        self.section_("override")





def makeWMStep(stepName):
    """
    _makeWMStep_

    Convienience method, instantiate a new WMStep with the name
    provided and wrap it in a helper

    """
    return WMStepHelper(WMStep(stepName))



#!/usr/bin/env python
"""
_ListCurrentSites_

This module list the current sites in Resource Control for Oracle
"""


from WMCore.ResourceControl.MySQL.ListCurrentSites import ListCurrentSites \
  as ListCurrentSitesMySQL
class ListCurrentSites(ListCurrentSitesMySQL):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Script to connect to a MongoDB instance based on a set of parameters given at runtime
Usage:
      source /data/srv/current/apps/reqmgr2ms/etc/profile.d/init.sh
      python3 -i -- mongoInit.py -c $WMCORE_SERVICE_CONFIG/reqmgr2ms-output/config-output.py
"""

import sys
import logging
import argparse

from WMCore.Database.MongoDB import MongoDB

from WMCore.Configuration import loadConfigurationFile

if __name__ == '__main__':

    FORMAT = "%(asctime)s:%(levelname)s:%(module)s:%(funcName)s(): %(message)s"
    logging.basicConfig(stream=sys.stdout, format=FORMAT, level=logging.INFO)
    logger = logging.getLogger(__name__)

    parser = argparse.ArgumentParser(
        prog='MongoInit',
        formatter_class=argparse.RawTextHelpFormatter,
        description=__doc__)

    parser.add_argument('-c', '--config', required=True,
                        help="""\
                        The path to MSCONFIG to be used, e.g.
                        for production: /data/srv/current/config/reqmgr2ms/config-output.py
                        """)
    args = parser.parse_args()

    logger.info("Loading configFile: %s", args.config)
    config = loadConfigurationFile(args.config)

    msConfig = config.section_('views').section_('data').dictionary_()

    msConfig.setdefault("mongoDBUrl", 'mongodb://localhost')
    msConfig.setdefault("mongoDBPort", 27017)
    msConfig.setdefault("mongoDB", 'mongoDBNull')
    msConfig.setdefault("mongoDBRetryCount", 3)
    msConfig.setdefault("mongoDBReplicaset", None)
    msConfig.setdefault("mockMongoDB", False)
    msConfig.setdefault("collection", None)

    mongoDBConfig = {
        'database': msConfig['mongoDB'],
        'server': msConfig['mongoDBUrl'],
        'port': msConfig['mongoDBPort'],
        'replicaset': msConfig['mongoDBReplicaset'],
        'logger': logger,
        'create': False,
        'mockMongoDB': msConfig['mockMongoDB']}

    mongoClt = MongoDB(**mongoDBConfig)
    mongoDB = getattr(mongoClt, msConfig['mongoDB'])
    mongoColl = mongoDB[msConfig['collection']] if msConfig['collection'] else None



#!/usr/bin/env python
"""
_GetWorkflowTask_

Oracle implementation of Jobs.GetWorkflowTask
"""

from WMCore.WMBS.MySQL.Jobs.GetWorkflowTask import GetWorkflowTask as MySQLGetWorkflowTask

class GetWorkflowTask(MySQLGetWorkflowTask):
    pass



#!/usr/bin/env python
"""
_SetStatus_

Oracle implementation for altering job status
"""


from WMCore.BossAir.MySQL.SetStatus import SetStatus as MySQLSetStatus

class SetStatus(MySQLSetStatus):
    """
    _SetStatus_

    Set the status of a list of jobs
    """



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#pylint: disable=W0212
"""
File       : TokenManager.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: IAM token manager
             by default it relies on the following environment/parameters:
             - IAM_TOKEN is either file name or actual token value
             - https://cms-auth.web.cern.ch/jwk is default CMS IAM provider
             - https://wlcg.cern.ch/jwt/v1/any is default for audience parameter
"""

# system modules
import os
import ssl
import time
import logging
import traceback

# third part library
try:
    import jwt
except ImportError:
    traceback.print_exc()
    jwt = None

from Utils.Utilities import encodeUnicodeToBytes

# prevent "SSL: CERTIFICATE_VERIFY_FAILED" error
# this will cause pylint warning W0212, therefore we ignore it above
ssl._create_default_https_context = ssl._create_unverified_context


def readToken(name=None):
    """
    Read IAM token either from environment or file name
    :param name: ether file name containing token or environment name which hold the token value.
    If not provided it will be assumed to read token from IAM_TOKEN environment.
    :return: token or None
    """
    if name and os.path.exists(name):
        token = None
        with open(name, 'r', encoding='utf-8') as istream:
            token = istream.read()
        return token
    if name:
        return os.environ.get(name)
    return os.environ.get("IAM_TOKEN")


def tokenData(token, url="https://cms-auth.web.cern.ch/jwk", audUrl="https://wlcg.cern.ch/jwt/v1/any"):
    """
    inspect and extract token data
    :param token: token string
    :param url: IAM provider URL
    :param audUrl: audience string
    """
    if not token or not jwt:
        return {}
    if isinstance(token, str):
        token = encodeUnicodeToBytes(token)
    jwksClient = jwt.PyJWKClient(url)
    signingKey = jwksClient.get_signing_key_from_jwt(token)
    key = signingKey.key
    headers = jwt.get_unverified_header(token)
    alg = headers.get('alg', 'RS256')
    data = jwt.decode(
        token,
        key,
        algorithms=[alg],
        audience=audUrl,
        options={"verify_exp": True},
        )
    return data


def isValidToken(token):
    """
    check if given token is valid or not

    :param token: token string
    :return: true or false
    """
    tokenDict = {}
    tokenDict = tokenData(token)
    exp = tokenDict.get('exp', 0)  # expire, seconds since epoch
    if not exp or exp < time.time():
        return False
    return True


class TokenManager():
    """
    TokenManager class handles IAM tokens
    """

    def __init__(self,
                 name=None,
                 url="https://cms-auth.web.cern.ch/jwk",
                 audUrl="https://wlcg.cern.ch/jwt/v1/any",
                 logger=None):
        """
        Token manager reads IAM tokens either from file or env.
        It caches token along with expiration timestamp.
        By default the env variable to use is IAM_TOKEN.
        :param name: string representing either file or env where we should read token from
        :param url: IAM provider URL
        :param audUrl: audience string
        :param logger: logger object or none to use default one
        """
        self.name = name
        self.url = url
        self.audUrl = audUrl
        self.expire = 0
        self.token = None
        self.logger = logger if logger else logging.getLogger()
        try:
            self.token = self.getToken()
        except Exception as exc:
            self.logger.exception("Failed to get token. Details: %s", str(exc))

    def getToken(self):
        """
        Return valid token and sets its expire timestamp
        """
        if not self.token or not isValidToken(self.token):
            self.token = readToken(self.name)
        tokenDict = {}
        try:
            tokenDict = tokenData(self.token, url=self.url, audUrl=self.audUrl)
            self.logger.debug(tokenDict)
        except Exception as exc:
            self.logger.exception(str(exc))
            raise
        self.expire = tokenDict.get('exp', 0)
        return self.token

    def getLifetime(self):
        """
        Return reamaining lifetime of existing token
        """
        return self.expire - int(time.time())



#!/usr/bin/env python

"""
The documentation for the framework
"""




from cherrypy import expose
from WMCore.WebTools.Page import TemplatedPage
from os import listdir

class Documentation(TemplatedPage):
    """
    The documentation for the framework
    """
    @expose
    def index(self):
        """
        The index of the documentation
        """
        templates = listdir(self.templatedir)
        index = "<h1>Documentation</h1>\n<ol>"
        for t in templates:
            if '.tmpl' in t:
                index = "%s\n<li><a href='%s'>%s</a></li>" % (index,
                                                      t.replace('.tmpl', ''),
                                                      t.replace('.tmpl', ''))
        index = "%s\n<li><a href='https://twiki.cern.ch/twiki/bin/view/CMS/DMWebtools'>twiki</a>" % (index)
        index = "%s\n<ol>" % (index)
        return index

    @expose
    def default(self, *args, **kwargs):
        """
        Show the documentation for a page or return the index
        """
        if len(args) > 0:
            return self.templatepage(args[0], config=self.config)
        else:
            return self.index()



#!/usr/bin/env python
"""
_GetValidation_

MySQL implementation of Subscription.GetValidation
"""

from WMCore.Database.DBFormatter import DBFormatter

class GetValidation(DBFormatter):
    sql = """SELECT wmbs_location.site_name, wmbs_subscription_validation.valid
                    FROM wmbs_location
               INNER JOIN wmbs_subscription_validation ON
                 wmbs_location.id = wmbs_subscription_validation.location_id AND
                 wmbs_subscription_validation.subscription_id = :id"""

    def execute(self, id, conn = None, transaction = False):
        results = self.dbi.processData(self.sql, {"id": id},
                                       transaction = transaction)
        formattedResults = self.formatDict(results)
        for formattedResult in formattedResults:
            if formattedResult["valid"] == 1:
                formattedResult = True
            else:
                formattedResult = False

        return formattedResults



#!/usr/bin/env python
"""
Adds PhEDEx Node Names into the WMBS database.
"""
from __future__ import division

from builtins import str, bytes

from WMCore.Database.DBFormatter import DBFormatter


class AddPNNs(DBFormatter):

    sql = "INSERT IGNORE INTO wmbs_pnns (pnn) VALUES (:pnn)"

    def execute(self, pnns, conn=None, transaction=False):
        """
        Adds either a single or a list of PNNs into WMBS
        """
        if isinstance(pnns, (str, bytes)):
            pnns = [pnns]

        binds = []
        for pnn in pnns:
            binds.append({'pnn': pnn})

        self.dbi.processData(self.sql, binds, conn=conn, transaction=transaction)

        return



#!/usr/bin/env python
"""
_CheckStatusForCompletedWorkflows_

Oracle implementation of DBSBuffer.CheckStatusForCompletedWorkflows
"""
from __future__ import print_function, division

from WMComponent.DBS3Buffer.MySQL.CheckStatusForCompletedWorkflows \
 import CheckStatusForCompletedWorkflows as MySQLCheckStatusForCompletedWorkflows

class CheckStatusForCompletedWorkflows(MySQLCheckStatusForCompletedWorkflows):
    """
    _CountFiles_

    """
    pass


#!/usr/bin/env python
"""
_WorkUnit_

Represents a work unit in WMBS
"""

from __future__ import absolute_import, division, print_function

import time

from WMCore.DataStructs.WorkUnit import WorkUnit as DSWorkUnit
from WMCore.WMBS.WMBSBase import WMBSBase

MAX_EVENT = 2 ** 31 - 1


class WorkUnit(WMBSBase, DSWorkUnit):
    """
    A simple object representing a file in WMBS
    """

    def __init__(self, wuid=None, taskID=None, retryCount=0, lastUnitCount=None, lastSubmitTime=int(time.time()),
                 status=0, firstEvent=1, lastEvent=MAX_EVENT, fileid=None, runLumi=None):
        WMBSBase.__init__(self)
        DSWorkUnit.__init__(self, taskID=taskID, retryCount=retryCount, lastUnitCount=lastUnitCount,
                            lastSubmitTime=lastSubmitTime, status=status, firstEvent=firstEvent, lastEvent=lastEvent,
                            fileid=fileid, runLumi=runLumi)
        self.setdefault('id', wuid)

    def exists(self):
        """
        If id is given, check with id or check with taskid, fileid, run/lumi
        """

        if self['id'] and self['id'] > 0:
            action = self.daofactory(classname='WorkUnit.ExistsByID')
            result = action.execute(wuid=self['id'], conn=self.getDBConn(), transaction=self.existingTransaction())
        elif self['taskid'] and self['fileid'] and self['run_lumi']:
            action = self.daofactory(classname='WorkUnit.ExistsByTaskFileLumi')
            result = action.execute(taskid=self['taskid'], fileid=self['fileid'], run_lumi=self['run_lumi'],
                                    conn=self.getDBConn(), transaction=self.existingTransaction())
        else:
            raise NotImplementedError("No way to look up existence without ID or task, file, run, and lumi")

        return result

    def getInfo(self):
        """
        Return the WorkUnit attributes as a tuple based off of DataStructs
        """
        dsInfo = super(WorkUnit, self).getInfo()
        return (self['id'],) + dsInfo

    def load(self):
        """
        _load_

        Load any meta data that is associated with a WorkUnit.
        """

        if self['id'] and self['id'] > 0:
            action = self.daofactory(classname="WorkUnit.GetByID")
            result = action.execute(self['id'], conn=self.getDBConn(), transaction=self.existingTransaction())
        elif self['taskid'] and self['fileid'] and self['run_lumi']:
            action = self.daofactory(classname='WorkUnit.GetByTaskFileLumi')
            result = action.execute(taskid=self['taskid'], fileid=self['fileid'],
                                    run_lumi=self['run_lumi'],
                                    conn=self.getDBConn(), transaction=self.existingTransaction())
        else:
            raise RuntimeError("Only methods to get a work unit is by ID or by task, fileid, run, lumi")

        self.update(result)
        return

    def create(self):
        """
        _create_

        Create a work unit.  If no transaction is passed in this will wrap all
        statements in a single transaction.
        """

        # TODO: Allow creation of a work unit without passing in ID

        existingTransaction = self.beginTransaction()

        if self.exists() is not False:
            self.commitTransaction(existingTransaction)
            self.load()
            # assume if the file already exist, parentage is already set.
            # or not exist yet
            return

        addAction = self.daofactory(classname="WorkUnit.Add")

        addAction.execute(taskid=self['taskid'], retry_count=self['retry_count'],
                          last_unit_count=self['last_unit_count'], last_submit_time=self['last_submit_time'],
                          status=self['status'],
                          first_event=self['firstevent'], last_event=self['lastevent'],
                          fileid=self['fileid'], run=self['run_lumi'].run, lumi=self['run_lumi'].lumis[0],
                          conn=self.getDBConn(), transaction=self.existingTransaction())

        self.commitTransaction(existingTransaction)

        return

    def delete(self):
        """
        Remove a WorkUnit from WMBS
        """
        action = self.daofactory(classname="WorkUnit.Delete")
        action.execute(wuid=self["id"], conn=self.getDBConn(), transaction=self.existingTransaction())
        return



#!/usr/bin/env python
"""
_JobStatusByLocation_

MySQL implementation for loading a job by scheduler status
"""

from future.utils import viewitems, viewvalues

from WMCore.Database.DBFormatter import DBFormatter

class JobStatusByLocation(DBFormatter):
    """
    _JobStatusByLocation_

    Load all jobs with a certain scheduler status including
    all the joined information.
    """


    sql = """SELECT wl.cms_name AS site_name, wl.pending_slots as pending_slots,
                    count(rj.wmbs_id) AS num_jobs,
                    st.name AS status, wl.plugin AS plugin
               FROM bl_runjob rj
               INNER JOIN bl_status st ON rj.sched_status = st.id
               INNER JOIN wmbs_job wj ON wj.id = rj.wmbs_id
               RIGHT OUTER JOIN wmbs_location wl ON wl.id = wj.location
               WHERE rj.status = :complete
               GROUP BY wl.cms_name, wl.plugin, st.name, wl.pending_slots
          """

    def mappedStatusFormat(self, results):
        """
        convert each individual batch system plugin status to a common status.
        Warning: This assumes all the plugin are under WMCore/BossAir/Plugins/
        and its module name and class name should be the same.

        __import__ doesn't reload if the module exist.
        (so doesn't need to keep track what is already imported.
        The performance difference should be small. If desired, maintain
        the cache to keep track imported plugIns
        """
        commonStates = {}
        for data in results:
            if data['site_name'] not in commonStates:
                commonStates[data['site_name']] = {}

            module = __import__("WMCore.BossAir.Plugins.%s" % data['plugin'],
                                globals(), locals(), [data['plugin']])
            plugIn = getattr(module, data['plugin'])

            for status in viewvalues(plugIn.stateMap()):
                commonStates[data['site_name']].setdefault(status, 0)

            state = plugIn.stateMap().get(data['status'])
            commonStates[data['site_name']][state] += data['num_jobs']
            commonStates[data['site_name']]['pending_slots'] = data['pending_slots']

        results = []
        for key, value in viewitems(commonStates):
            reformedData = {'site_name': key}
            reformedData.update(value)
            results.append(reformedData)

        return results


    def execute(self, commonFormat = True, conn = None, transaction = False):
        """
        _execute_

        Load all jobs either running or not (running by default)
        """
        complete = '1'
        binds = {'complete': complete}


        result = self.dbi.processData(self.sql, binds, conn = conn,
                                      transaction = transaction)
        if commonFormat:
            return self.mappedStatusFormat(self.formatDict(result))
        else:
            return self.formatDict(result)



#! /usr/bin/env python

"""
_MiscAlgos_

Useful little tools that can be handy anywhere.
"""
import logging

def sortListByKey(data, key):
    """
    Return list of dictionaries as a
    dictionary of lists of dictionaries
    keyed by one original key

    """
    final = {}

    for entry in data:
        value = entry.get(key, None)
        if value == None:
            # Empty dict value?
            # This is an error, but we can't handle it here
            logging.error("Found entry with no key in sortListByKey: %s", entry)
            logging.error("Skipping")
            continue
        if isinstance(value, set):
            try:
                v     = value
                value = v.pop()
                v.add(value)
            except KeyError:
                # Set was empty?
                # This is peculiar, we can't handle this.
                logging.error("Found list entry with empty key set in sortListByKey: %s", entry)
                logging.error("Skipping")
                continue
        if value not in final:
            final[value] = []
        final[value].append(entry)

    return final



## {{{ http://code.activestate.com/recipes/576644/ (r1)
def dict_diff(first, second):
    """ Return a dict of keys that differ with another config object.  If a value is
        not found in one fo the configs, it will be represented by KEYNOTFOUND.
        @param first:   Fist dictionary to diff.
        @param second:  Second dicationary to diff.
        @return diff:   Dict of Key => (first.val, second.val)
    """
    KEYNOTFOUND = '<KEYNOTFOUND>'
    diff = {}
    # Check all keys in first dict
    for key in first:
        if (key not in second):
            diff[key] = (first[key], KEYNOTFOUND)
        elif (first[key] != second[key]):
            diff[key] = (first[key], second[key])
    # Check all keys in second dict to find missing
    for key in second:
        if (key not in first):
            diff[key] = (KEYNOTFOUND, second[key])
    return diff
## end of http://code.activestate.com/recipes/576644/ }}}



#!/usr/bin/env python
"""
_WMObject_

Helper class that other objects should inherit from

"""
from builtins import object
__all__ = []




class WMObject(object):
    """
    Helper class that other objects should inherit from
    """
    def __init__(self, config = {}):
        #Config is a WMCore.Configuration
        self.config = config

    def makelist(self, thelist):
        """
        Simple method to ensure thelist is a list
        """
        if isinstance(thelist, set):
            thelist = list(thelist)
        elif not isinstance(thelist, list):
            thelist = [thelist]
        return thelist

    def makeset(self, theset):
        """
        Simple method to ensure theset is a set
        """
        if not isinstance(theset, set):
            theset = set(self.makelist(theset))
        return theset

    def flatten(self, list):
        """
        If a list has only one element return just that element, otherwise
        return the original list
        """
        if len(list) == 1:
            return list[0]
        return list



#!/usr/bin/env python
"""
_GetAvailableFilesMeta_

Oracle implementation of Subscription.GetAvailableFilesMeta
"""

from WMCore.WMBS.MySQL.Subscriptions.GetAvailableFilesMeta import \
     GetAvailableFilesMeta as GetAvailableFilesMetaMySQL

class GetAvailableFilesMeta(GetAvailableFilesMetaMySQL):
    pass



#!/usr/bin/env python

"""
Defines default config values for JobStatusLite specific parameters.
"""

import os

from WMCore.Agent.Configuration import Configuration

__all__ = []

config = Configuration()
config.component_('JobStatusLite')
config.JobStatusLite.namespace = 'WMComponent.JobStatusLite.JobStatusLite'
config.JobStatusLite.componentDir = os.path.join(os.getcwd(), 'Components')
config.JobStatusLite.logLevel = 'INFO'
config.JobStatusLite.pollInterval = 180
config.JobStatusLite.queryInterval = 120
config.JobStatusLite.jobLoadLimit = 100
config.JobStatusLite.maxJobQuery = 100
config.JobStatusLite.taskLimit = 30
config.JobStatusLite.maxJobsCommit = 100
config.JobStatusLite.processes = 5



#!/usr/bin/env python
"""
_GeneratorFactory_


"""

from WMCore.WMException import WMException
from WMCore.WMFactory import WMFactory


class GeneratorFactoryException(WMException):
    """
    _GeneratorFactoryException_

    Exception for missing objects or problems

    """
    pass


class GeneratorFactory(WMFactory):
    """
    _GeneratorFactory_

    Instantiate a WMFactory instance with the appropriate namespace

    """

    def __init__(self):
        self.factory = WMFactory(self.__class__.__name__,
                                 "WMCore.JobSplitting.Generators")

    def getGenerator(self, generatorName, **options):
        """
        _getGenerator_

        factory method to return a step template instance based on the
        name of the step

        """
        args = {}
        args.update(options)
        try:
            return self.factory.loadObject(generatorName,
                                           args)
        except WMException as wmEx:
            msg = "GeneratorFactory Unable to load Object: %s" % generatorName
            raise GeneratorFactoryException(msg)
        except Exception as ex:
            msg = "Error creating object %s in GeneratorFactory:\n" % generatorName
            msg += str(ex)
            raise GeneratorFactoryException(msg)
        return

    def makeGenerators(self, wmTaskHelper):
        """
        _makeGenerators_

        Util function to build the set of generators defined
        by the task provided

        """
        result = []
        for generator in wmTaskHelper.listGenerators():
            genInstance = self.getGenerator(generator, **wmTaskHelper.getGeneratorSettings(generator))
            result.append(genInstance)
        return result



#!/usr/bin/env python
"""
_GetSiteInfo_

MySQL implementation of Locations.GetSiteInfo
"""

from WMCore.Database.DBFormatter import DBFormatter


class GetSiteInfo(DBFormatter):
    """
    Grab all the relevant information for a given site.
    Usually useful only in the submitter
    """
    sql = """SELECT wl.site_name, wpnn.pnn, wl.ce_name, wl.pending_slots,
                    wl.running_slots, wl.plugin, wl.cms_name, wlst.name AS state
               FROM wmbs_location wl
                 INNER JOIN wmbs_location_pnns wls ON wls.location = wl.id
                 INNER JOIN wmbs_pnns wpnn ON wpnn.id = wls.pnn
                 INNER JOIN wmbs_location_state wlst ON wlst.id = wl.state
          """

    def execute(self, siteName=None, conn=None, transaction=False):
        if not siteName:
            results = self.dbi.processData(self.sql, conn=conn,
                                           transaction=transaction)
        else:
            sql = self.sql + " WHERE wl.site_name = :site"
            results = self.dbi.processData(sql, {'site': siteName},
                                       conn=conn, transaction=transaction)
        return self.formatDict(results)



#!/usr/bin/env python
"""
_ListIncomplete_

Oracle implementation of Subscription.ListIncomplete
"""

from WMCore.WMBS.MySQL.Subscriptions.ListIncomplete import ListIncomplete as ListIncompleteMySQL

class ListIncomplete(ListIncompleteMySQL):
    pass



#!/usr/bin/env python
"""
_FailedJobsByWorkflow_

MySQL implementation of Workflow.FailedJobs
"""

from WMCore.Database.DBFormatter import DBFormatter

class FailedJobsByWorkflow(DBFormatter):
    sql = """SELECT ww.id, ww.name, ww.task, wj.id as job_id
                    FROM wmbs_workflow ww
               INNER JOIN wmbs_subscription ws ON
                 ww.id = ws.workflow
               LEFT OUTER JOIN wmbs_jobgroup wjg ON
                 ws.id = wjg.subscription
               LEFT OUTER JOIN wmbs_job wj ON
                 wjg.id = wj.jobgroup
               LEFT OUTER JOIN wmbs_job_state wjs ON
                 wj.state = wjs.id
             WHERE wj.outcome = 0 AND
                   (wjs.name = 'exhausted' OR
                    wjs.name = 'cleanout') AND
                    ww.name = :workflowName
             ORDER BY ww.id"""

    def execute(self, workflowName, conn = None, transaction = False):
        result = self.dbi.processData(self.sql, {'workflowName': workflowName},
                                      conn = conn, transaction = transaction)
        return self.formatDict(result)



#!/usr/bin/env python
"""
_ListSitesSlotsState_

Query the Resource Control database for Oracle to get all the current sites,
pending slots, running slots and state.
"""

from WMCore.ResourceControl.MySQL.ListSitesSlotsState import ListSitesSlotsState \
  as ListSitesSlotsStateMySQL
class ListSitesSlotsState(ListSitesSlotsStateMySQL):
    pass



#!/usr/bin/env python
"""
_GetUninjectedFiles_

Oracle implementation on PhEDExInjector.GetUninjectedFiles.
"""
from __future__ import division

from WMComponent.RucioInjector.Database.MySQL.GetUninjectedFiles import GetUninjectedFiles as MySQLBase


class GetUninjectedFiles(MySQLBase):
    """
    _GetUninjectedFiles_

    """
    pass



#!/usr/bin/env python
"""
_CheckInjectedWorkflow_

Oracle implementation of Workflow.CheckInjectedWorkflow
"""

from WMCore.WMBS.MySQL.Workflow.CheckInjectedWorkflow import CheckInjectedWorkflow as MySQLCheckInjectedWorkflow

class CheckInjectedWorkflow(MySQLCheckInjectedWorkflow):
    """
    Oracle version

    """



"""
_PeriodicSiblingComplete_

MySQL implementation of JobSplitting.PeriodicSiblingComplete
"""

import time

from WMCore.Database.DBFormatter import DBFormatter

class PeriodicSiblingComplete(DBFormatter):
    """
    _PeriodicSiblingComplete_

    Check if all other subscriptions on the same input fileset
    as the specified subscription are complete (ie. have no
    available or acquiered files and the fileset is closed).
    Fileset closed is assumed for the query as the Harvest
    job splitter only does this check if it is.

    Returns either 0 (sibling subscription is not complete)
                or 1 (sibling subscription is complete).

    """

    sql = """SELECT CASE
                      WHEN COUNT(wmbs_sub_files_available.subscription) > 0 THEN 0
                      WHEN COUNT(wmbs_sub_files_acquired.subscription) > 0 THEN 0
                      ELSE 1
                    END
             FROM wmbs_subscription
             INNER JOIN wmbs_fileset ON
               wmbs_fileset.id = wmbs_subscription.fileset
             LEFT OUTER JOIN wmbs_subscription sibling_subscription ON
               sibling_subscription.fileset = wmbs_fileset.id AND
               sibling_subscription.id != wmbs_subscription.id
             LEFT OUTER JOIN wmbs_sub_files_available ON
               wmbs_sub_files_available.subscription = sibling_subscription.id
             LEFT OUTER JOIN wmbs_sub_files_acquired ON
               wmbs_sub_files_acquired.subscription = sibling_subscription.id
             WHERE wmbs_subscription.id = :subscription
             """

    def execute(self, subscription, conn = None, transaction = False):

        binds = { 'SUBSCRIPTION' : subscription }

        complete = self.dbi.processData(self.sql, binds, conn = conn,
                                        transaction = transaction)[0].fetchall()[0][0]

        return (complete == 1)



#!/usr/bin/env python
"""
_XRDCPImpl_

Implementation of StageOutImpl interface for xrdcp

Generic, will/should work with any site.

"""
from __future__ import print_function

import argparse
import logging
import os

from WMCore.Storage.Registry import registerStageOutImpl
from WMCore.Storage.StageOutImpl import StageOutImpl


class XRDCPImpl(StageOutImpl):
    """
    _XRDCPImpl_

    """

    def __init__(self, stagein=False):
        StageOutImpl.__init__(self, stagein)
        self.numRetries = 5
        self.retryPause = 300
        self.xrdfsCmd = "xrdfs"

    def createSourceName(self, protocol, pfn):
        """
        _createSourceName_

        """
        return pfn

    def createOutputDirectory(self, targetPFN):
        """
        _createOutputDirectory_

        not needed since xrdcp does it automatically
        """
        return

    def _checkXRDUtilsExist(self):
        """
        Verifies whether xrdcp and xrdfs utils exist in the job path.

        :return: True if both exist, otherwise False
        """
        foundXrdcp, foundXrdfs = False, False
        for path in os.environ["PATH"].split(os.pathsep):
            if os.access(os.path.join(path, "xrdcp"), os.X_OK):
                foundXrdcp = True
            if os.access(os.path.join(path, "xrdfs"), os.X_OK):
                foundXrdfs = True

        return foundXrdcp & foundXrdfs

    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None):
        """
        _createStageOutCommand_

        Build the actual xrdcp stageout command

        If adler32 checksum is provided, use it for the transfer
        xrdcp options used:
          --force : re-creates a file if it's already present
          --nopbar : does not display the progress bar

        """
        if not options:
            options = ''

        parser = argparse.ArgumentParser()
        parser.add_argument('--wma-cerncastor', action='store_true')
        parser.add_argument('--wma-disablewriterecovery', action='store_true')
        parser.add_argument('--wma-preload')
        args, unknown = parser.parse_known_args(options.split())

        # strip out WMAgent specific options
        unknown = [option for option in unknown if not option.startswith('--wma-')]
        copyCommandOptions = ' '.join(unknown)

        copyCommand = ""

        if self.stageIn:
            remotePFN, localPFN = sourcePFN, targetPFN
        else:
            remotePFN, localPFN = targetPFN, sourcePFN
            copyCommand += "LOCAL_SIZE=`stat -c%%s \"%s\"`\n" % localPFN
            copyCommand += "echo \"Local File Size is: $LOCAL_SIZE\"\n"
            if args.wma_cerncastor:
                targetPFN += "?svcClass=t0cms"

        useChecksum = (checksums is not None and 'adler32' in checksums and not self.stageIn)

        # If not, check whether OSG cvmfs repo is available.
        # This likely only works on RHEL7 x86-64 (and compatible) OS, but this
        # represents most of our resources and it's a fallback mechanism anyways
        if not self._checkXRDUtilsExist():
            logging.warning("Failed to find XRootD in the path. Trying fallback to OSG CVMFS...")
            initFile = "/cvmfs/oasis.opensciencegrid.org/osg-software/osg-wn-client/current/el7-x86_64/setup.sh"
            if os.path.isfile(initFile):
                copyCommand += "source %s\n" % initFile

        if args.wma_disablewriterecovery:
            copyCommand += "env XRD_WRITERECOVERY=0 "

        if args.wma_preload:
            xrdcpCmd = "%s xrdcp" % args.wma_preload
            self.xrdfsCmd = "%s xrdfs" % args.wma_preload
        else:
            xrdcpCmd = "xrdcp"
            self.xrdfsCmd = "xrdfs"

        copyCommand += "%s --force --nopbar " % xrdcpCmd

        if copyCommandOptions:
            copyCommand += "%s " % copyCommandOptions

        if useChecksum:
            checksums['adler32'] = "%08x" % int(checksums['adler32'], 16)
            copyCommand += "--cksum adler32:%s " % checksums['adler32']

        copyCommand += " \"%s\" " % sourcePFN
        copyCommand += " \"%s\" \n" % targetPFN

        if self.stageIn:
            copyCommand += "LOCAL_SIZE=`stat -c%%s \"%s\"`\n" % localPFN
            copyCommand += "echo \"Local File Size is: $LOCAL_SIZE\"\n"

        (_, host, path, _) = self.splitPFN(remotePFN)

        if self.stageIn:
            removeCommand = ""
        else:
            removeCommand = "%s '%s' rm %s ;" % (self.xrdfsCmd, host, path)

        copyCommand += "REMOTE_SIZE=`%s '%s' stat '%s' | grep Size | sed -r 's/.*Size:[ ]*([0-9]+).*/\\1/'`\n" % (self.xrdfsCmd, host, path)
        copyCommand += "echo \"Remote File Size is: $REMOTE_SIZE\"\n"

        if useChecksum:

            copyCommand += "echo \"Local File Checksum is: %s\"\n" % checksums['adler32']
            copyCommand += "REMOTE_XS=`%s '%s' query checksum '%s' | grep -i adler32 | sed -r 's/.*[adler|ADLER]32[ ]*([0-9a-fA-F]{8}).*/\\1/'`\n" % (self.xrdfsCmd, host, path)
            copyCommand += "echo \"Remote File Checksum is: $REMOTE_XS\"\n"

            copyCommand += "if [ $REMOTE_SIZE ] && [ $REMOTE_XS ] && [ $LOCAL_SIZE == $REMOTE_SIZE ] && [ '%s' == $REMOTE_XS ]; then exit 0; " % checksums['adler32']
            copyCommand += "else echo \"ERROR: Size or Checksum Mismatch between local and SE\"; %s exit 60311 ; fi" % removeCommand

        else:

            copyCommand += "if [ $REMOTE_SIZE ] && [ $LOCAL_SIZE == $REMOTE_SIZE ]; then exit 0; "
            copyCommand += "else echo \"ERROR: Size Mismatch between local and SE\"; %s exit 60311 ; fi" % removeCommand

        return copyCommand

    def removeFile(self, pfnToRemove):
        """
        _removeFile_

        """
        (_, host, path, _) = self.splitPFN(pfnToRemove)
        command = "%s %s rm %s" % (self.xrdfsCmd, host, path)
        self.executeCommand(command)
        return

registerStageOutImpl("xrdcp", XRDCPImpl)



#!/usr/bin/env python
"""
WorkQueue SplitPolicyInterface

"""
from builtins import str as newstr, bytes
from future.utils import viewitems


__all__ = []
from WMCore.WorkQueue.Policy.PolicyInterface import PolicyInterface
from WMCore.WorkQueue.DataStructs.WorkQueueElement import WorkQueueElement
from WMCore.DataStructs.LumiList import LumiList
from WMCore.WorkQueue.WorkQueueExceptions import WorkQueueWMSpecError, WorkQueueNoWorkError
from dbs.exceptions.dbsClientException import dbsClientException
from WMCore.Services.CRIC.CRIC import CRIC
from WMCore.Services.Rucio.Rucio import Rucio
from WMCore.Services.DBS.DBSErrors import DBSReaderError
from WMCore import Lexicon


class StartPolicyInterface(PolicyInterface):
    """Interface for start policies"""

    def __init__(self, **args):
        # We need to pop this object instance from args because otherwise
        # the super class blows up when doing a deepcopy(args)
        self.rucio = args.pop("rucioObject", None)
        PolicyInterface.__init__(self, **args)
        self.workQueueElements = []
        self.wmspec = None
        self.team = None
        self.initialTask = None
        self.splitParams = None
        self.dbs_pool = {}
        self.data = {}
        self.lumi = None
        self.couchdb = None
        self.rejectedWork = []  # List of inputs that were rejected
        self.badWork = []  # list of bad work unit (e.g. without any valid files)
        self.pileupData = {}
        self.cric = CRIC()
        # FIXME: for the moment, it will always use the default value
        self.rucioAcct = self.args.get("rucioAcct", "wmcore_transferor")
        if not self.rucio:
            self.rucio = Rucio(self.rucioAcct, configDict={'logger': self.logger})

    def split(self):
        """Apply policy to spec"""
        raise NotImplementedError

    def validate(self):
        """Check params and spec are appropriate for the policy"""
        raise NotImplementedError

    def validateCommon(self):
        """Common validation stuff"""
        try:
            Lexicon.requestName(self.wmspec.name())
        except Exception as ex:  # can throw many errors e.g. AttributeError, AssertionError etc.
            error = WorkQueueWMSpecError(self.wmspec, "Workflow name validation error: %s" % str(ex))
            raise error

        if self.initialTask.siteWhitelist():
            if isinstance(self.initialTask.siteWhitelist(), (newstr, bytes)):
                error = WorkQueueWMSpecError(self.wmspec, 'Invalid site whitelist: Must be tuple/list but is %s' % type(
                    self.initialTask.siteWhitelist()))
                raise error
            try:
                [Lexicon.cmsname(site) for site in self.initialTask.siteWhitelist()]
            except Exception as ex:  # can throw many errors e.g. AttributeError, AssertionError etc.
                error = WorkQueueWMSpecError(self.wmspec, "Site whitelist validation error: %s" % str(ex))
                raise error
        else:
            error = WorkQueueWMSpecError(self.wmspec, "Site whitelist validation error: Empty site whitelist")
            raise error

        if self.initialTask.siteBlacklist():
            if isinstance(self.initialTask.siteBlacklist(), (newstr, bytes)):
                error = WorkQueueWMSpecError(self.wmspec, 'Invalid site blacklist: Must be tuple/list but is %s' % type(
                    self.initialTask.siteBlacklist()))
                raise error
            try:
                [Lexicon.cmsname(site) for site in self.initialTask.siteBlacklist()]
            except Exception as ex:  # can throw many errors e.g. AttributeError, AssertionError etc.
                error = WorkQueueWMSpecError(self.wmspec, "Site blacklist validation error: %s" % str(ex))
                raise error

        # splitter settings
        if self.args.get('SliceSize', 1) <= 0:
            error = WorkQueueWMSpecError(self.wmspec, 'Zero or negative SliceSize parameter')
            raise error
        if self.args.get('SubSliceSize', 1) <= 0:
            error = WorkQueueWMSpecError(self.wmspec, 'Zero or negative SubSliceSize parameter')
            raise error

        # check input dataset is valid
        try:
            if self.initialTask.getInputDatasetPath():
                Lexicon.dataset(self.initialTask.getInputDatasetPath())
        except Exception as ex:  # can throw many errors e.g. AttributeError, AssertionError etc.
            error = WorkQueueWMSpecError(self.wmspec, "Dataset validation error: %s" % str(ex))
            raise error

        # if pileup is found, check that they are valid datasets
        try:
            pileupDatasets = self.wmspec.listPileupDatasets()
            for dbsUrl in pileupDatasets:
                for dataset in pileupDatasets[dbsUrl]:
                    Lexicon.dataset(dataset)
        except Exception as ex:  # can throw many errors e.g. AttributeError, AssertionError etc.
            error = WorkQueueWMSpecError(self.wmspec, "Pileup dataset validation error: %s" % str(ex))
            raise error

    def newQueueElement(self, **args):
        # DBS Url may not be available in the initial task
        # but in the pileup data (MC pileup)
        dbsUrl = self.initialTask.dbsUrl()
        if dbsUrl is None and self.pileupData:
            # Get the first DBS found
            dbsUrl = next(iter(self.wmspec.listPileupDatasets()))

        args.setdefault('Status', 'Available')
        args.setdefault('WMSpec', self.wmspec)
        args.setdefault('Task', self.initialTask)
        args.setdefault('RequestName', self.wmspec.name())
        args.setdefault('TaskName', self.initialTask.name())
        args.setdefault('Dbs', dbsUrl)
        args.setdefault('SiteWhitelist', self.initialTask.siteWhitelist())
        args.setdefault('SiteBlacklist', self.initialTask.siteBlacklist())
        args.setdefault('StartPolicy', self.wmspec.startPolicy())
        args.setdefault('EndPolicy', self.wmspec.endPolicyParameters())
        args.setdefault('Priority', self.wmspec.priority())
        args.setdefault('PileupData', self.pileupData)
        if not args['Priority']:
            args['Priority'] = 0
        ele = WorkQueueElement(**args)
        for data, sites in viewitems(ele['Inputs']):
            if not sites:
                raise WorkQueueWMSpecError(self.wmspec, 'Input data has no locations "%s"' % data)

        # catch infinite splitting loops
        if len(self.workQueueElements) > self.args.get('maxRequestSize', 1e8):
            raise WorkQueueWMSpecError(self.wmspec, 'Too many elements (%d)' % self.args.get('MaxRequestElements', 1e8))
        self.workQueueElements.append(ele)

    def __call__(self, wmspec, task, data=None, mask=None, team=None, continuous=False, rucioObj=None):
        self.wmspec = wmspec
        # bring in spec specific settings
        self.args.update(self.wmspec.startPolicyParameters())
        self.initialTask = task
        if data:
            self.data = data
        self.mask = mask
        self.validate()
        try:
            pileupDatasets = self.wmspec.listPileupDatasets()
            if pileupDatasets:
                self.pileupData = self.getDatasetLocations(pileupDatasets)
            self.split()
        # For known exceptions raise custom error that will fail the workflow.
        except dbsClientException as ex:
            # A dbs configuration error implies the spec is invalid
            error = WorkQueueWMSpecError(self.wmspec, "DBS config error: %s" % str(ex))
            raise error
        except AssertionError as ex:
            # Assertion generally means validation of an input field failed
            error = WorkQueueWMSpecError(self.wmspec, "Assertion error: %s" % str(ex))
            raise error
        except DBSReaderError as ex:
            # Hacky way of identifying non-existant data, DbsBadRequest chomped by DBSReader
            if 'Invalid parameters' in str(ex):
                data = task.data.input.pythonise_() if task.data.input else 'None'
                msg = """data: %s, mask: %s, pileup: %s. %s""" % (str(data), str(mask), str(pileupDatasets), str(ex))
                error = WorkQueueNoWorkError(self.wmspec, msg)
                raise error
            raise  # propagate other dbs errors

        # if we have no new elements and we are not adding work to request
        # already running, then raise exception
        if not self.workQueueElements and not continuous:
            data = task.data.input.pythonise_() if task.data.input else 'None'
            msg = "Failed to add work. Input data: %s, mask: %s." % (str(data), str(mask))
            error = WorkQueueNoWorkError(self.wmspec, msg)
            raise error

        return self.workQueueElements, self.rejectedWork, self.badWork

    def dbs(self, dbs_url=None):
        """Get DBSReader"""
        from WMCore.WorkQueue.WorkQueueUtils import get_dbs
        if dbs_url is None:
            dbs_url = self.initialTask.dbsUrl()
        return get_dbs(dbs_url)

    @staticmethod
    def supportsWorkAddition():
        """Indicates if a given policy supports addition of new work"""
        return False

    def getMaskedBlocks(self, task, dbs, datasetPath):
        """
        Get the blocks which pass the lumi mask restrictions. For each block
        return the list of lumis which were ok (given the lumi mask). The data
        structure returned is the following:
        {
            "block1" : {"file1" : LumiList(), "file5" : LumiList(), ...}
            "block2" : {"file2" : LumiList(), "file7" : LumiList(), ...}
        }
        """
        # Get the task mask as a LumiList object to make operations easier
        maskedBlocks = {}
        taskMask = task.getLumiMask()

        # for performance reasons, we first get all the blocknames
        blocks = [x['block_name'] for x in dbs.dbs.listBlocks(dataset=datasetPath)]

        for block in blocks:
            fileLumis = dbs.dbs.listFileLumis(block_name=block, validFileOnly=1)
            for fileLumi in fileLumis:
                lfn = fileLumi['logical_file_name']
                runNumber = str(fileLumi['run_num'])
                lumis = fileLumi['lumi_section_num']
                fileMask = LumiList(runsAndLumis={runNumber: lumis})
                commonMask = taskMask & fileMask
                if commonMask:
                    maskedBlocks.setdefault(block, {})
                    maskedBlocks[block].setdefault(lfn, LumiList())
                    maskedBlocks[block][lfn] += commonMask

        return maskedBlocks

    def modifyPolicyForWorkAddition(self, inboxElement):
        """Set modifiers to the policy based on the inboxElement information so that after a splitting pass
        with this policy strictly new work is returned, the inbox element must have information
        about already existing work"""
        raise NotImplementedError("This can't be called on a base StartPolicyInterface object")

    def newDataAvailable(self, task, inbound):
        """
            Returns True if there is data in the future could be included as an element
            for the inbound parent. However it doesn't guarantee that the new data
            will be included if the inbound element is split (i.e. the new data could be open blocks for the Block policy).
        """
        raise NotImplementedError("This can't be called on a base StartPolicyInterface object")

    def getDatasetLocations(self, datasets):
        """
        Returns a dictionary with the location of the datasets according to Rucio
        The definition of "location" here is a union of all sites holding at least
        part of the dataset (defined by the DATASET grouping).
        :param datasets: dictionary with a list of dataset names (key'ed by the DBS URL)
        :return: a dictionary of dataset locations, key'ed by the dataset name
        """
        result = {}
        for dbsUrl in datasets:
            for datasetPath in datasets[dbsUrl]:
                locations = self.rucio.getDataLockedAndAvailable(name=datasetPath,
                                                                 account=self.rucioAcct)
                result[datasetPath] = self.cric.PNNstoPSNs(locations)
        return result

    def blockLocationRucioPhedex(self, blockName):
        """
        Wrapper around Rucio and PhEDEx systems.
        Fetch the current location of the block name (if Rucio,
        also consider the locks made on that block)
        :param blockName: string with the block name
        :return: a list of RSEs
        """
        location = self.rucio.getDataLockedAndAvailable(name=blockName,
                                                        account=self.rucioAcct)
        return location


#!/usr/bin/env python
"""
Oracle implementation of File.SetParentage

Make the parentage link between two file lfns in bulk
"""

from WMCore.WMBS.MySQL.Files.SetParentage import SetParentage as MySQLSetParentage

class SetParentage(MySQLSetParentage):
    sql = """INSERT INTO wmbs_file_parent (child, parent)
               SELECT
                 (SELECT id FROM wmbs_file_details WHERE lfn = :child),
                 (SELECT id FROM wmbs_file_details WHERE lfn = :parent) FROM DUAL
               WHERE NOT EXISTS
                 (SELECT child FROM wmbs_file_parent WHERE
                    child = (SELECT id FROM wmbs_file_details WHERE lfn = :child) AND
                    parent = (SELECT id FROM wmbs_file_details WHERE lfn = :parent))"""



#!/usr/bin/env python
"""
_GetRunLumiFile_

MySQL implementation of GetRunLumiFile
"""




from WMCore.Database.DBFormatter import DBFormatter

class GetRunLumiFile(DBFormatter):
    sql = """SELECT flr.run AS run, flr.lumi AS lumi, flr.num_events as num_events
               FROM wmbs_file_runlumi_map flr
               INNER JOIN wmbs_file_details wfd ON wfd.id = flr.fileid
               WHERE wfd.lfn = :lfn"""


    def getBinds(self, file=None):
        binds = []
        file = self.dbi.makelist(file)
        for f in file:
            binds.append({'lfn': f})
        return binds

    def format(self, result):
        "Return a list of Run/Lumi Set"
        run_lumis={}
        for r in result:
            for i in r.fetchall():
                if i[0] not in run_lumis:
                    run_lumis[i[0]]=[]
                run_lumis[i[0]].append((i[1], i[2]))
                r.close()
        return run_lumis

    def execute(self, file=None, conn = None, transaction = False):
        binds = self.getBinds(file)

        result = self.dbi.processData(self.sql, binds,
                         conn = conn, transaction = transaction)
        return self.format(result)



#!/usr/bin/env python
"""
_ListAlgoDatasetAssoc_

Oracle implementation of DBSBuffer.ListAlgoDatasetAssoc
"""




from WMComponent.DBS3Buffer.MySQL.ListAlgoDatasetAssoc import ListAlgoDatasetAssoc as MySQLListAlgoDatasetAssoc

class ListAlgoDatasetAssoc(MySQLListAlgoDatasetAssoc):
    """
    _ListAlgoDatasetAssoc_

    Retrieve information about a dataset/algorthim association in the DBSBuffer.
    This is mostly used by the unit tests.
    """
    pass
